# Copyright 2017 Spotify AB
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import namedtuple
import cstar.exceptions


class ExecutionResult(namedtuple("ExecutionResult", "command status out err")):

    def out_or_raise(self):
        if self.status != 0:
            raise cstar.exceptions.FailedExecution('Command "%s" failed with status %d and error message %s' %
                                                   (self.command, self.status, self.err))
        return self.out
